"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySqlConnection = void 0;
var tslib_1 = require("tslib");
var promise_1 = tslib_1.__importDefault(require("mysql2/promise"));
var MySqlConnection = /** @class */ (function () {
    function MySqlConnection(dbUrl, sessionStorageIdentifier, connectionPoolLimit) {
        this.dbUrl = dbUrl;
        this.connectionPoolLimit = connectionPoolLimit;
        this.ready = this.init();
        this.sessionStorageIdentifier = sessionStorageIdentifier;
    }
    MySqlConnection.prototype.query = function (query, params) {
        if (params === void 0) { params = []; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.pool.query(query, params)];
                }
            });
        });
    };
    /**
     * Runs a series of queries in a transaction.
     *
     * @param queries an array of SQL queries to execute in a transaction
     */
    MySqlConnection.prototype.transaction = function (queries) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var client, queries_1, queries_1_1, query, e_1_1, error_1;
            var e_1, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _b.sent();
                        // check if the first and last queries are BEGIN and COMMIT, if they are, ignore them
                        // mysql2
                        if (queries[0] === 'BEGIN') {
                            queries.shift();
                        }
                        if (queries[queries.length - 1] === 'COMMIT') {
                            queries.pop();
                        }
                        return [4 /*yield*/, this.pool.getConnection()];
                    case 2:
                        client = _b.sent();
                        _b.label = 3;
                    case 3:
                        _b.trys.push([3, 14, 16, 17]);
                        return [4 /*yield*/, client.beginTransaction()];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5:
                        _b.trys.push([5, 10, 11, 12]);
                        queries_1 = tslib_1.__values(queries), queries_1_1 = queries_1.next();
                        _b.label = 6;
                    case 6:
                        if (!!queries_1_1.done) return [3 /*break*/, 9];
                        query = queries_1_1.value;
                        return [4 /*yield*/, client.query(query)];
                    case 7:
                        _b.sent();
                        _b.label = 8;
                    case 8:
                        queries_1_1 = queries_1.next();
                        return [3 /*break*/, 6];
                    case 9: return [3 /*break*/, 12];
                    case 10:
                        e_1_1 = _b.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 12];
                    case 11:
                        try {
                            if (queries_1_1 && !queries_1_1.done && (_a = queries_1.return)) _a.call(queries_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 12: return [4 /*yield*/, client.commit()];
                    case 13:
                        _b.sent();
                        return [3 /*break*/, 17];
                    case 14:
                        error_1 = _b.sent();
                        // rollback if any of the queries fail
                        return [4 /*yield*/, client.rollback()];
                    case 15:
                        // rollback if any of the queries fail
                        _b.sent();
                        throw error_1;
                    case 16:
                        client.release();
                        return [7 /*endfinally*/];
                    case 17: return [2 /*return*/];
                }
            });
        });
    };
    MySqlConnection.prototype.connect = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        // Nothing else to do here
                        return [2 /*return*/, Promise.resolve()];
                }
            });
        });
    };
    MySqlConnection.prototype.disconnect = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.pool.end()];
                    case 2:
                        _a.sent();
                        this.ready = this.init();
                        return [2 /*return*/];
                }
            });
        });
    };
    MySqlConnection.prototype.getDatabase = function () {
        return decodeURIComponent(this.dbUrl.pathname.slice(1));
    };
    MySqlConnection.prototype.hasTable = function (tablename) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var query, _a, rows;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.ready];
                    case 1:
                        _b.sent();
                        query = "\n      SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES\n        WHERE TABLE_NAME = ".concat(this.getArgumentPlaceholder(), "\n          AND TABLE_SCHEMA = ").concat(this.getArgumentPlaceholder(), ";\n    ");
                        return [4 /*yield*/, this.pool.query(query, [
                                tablename,
                                this.getDatabase(),
                            ])];
                    case 2:
                        _a = tslib_1.__read.apply(void 0, [_b.sent(), 1]), rows = _a[0];
                        return [2 /*return*/, Array.isArray(rows) && rows.length === 1];
                }
            });
        });
    };
    MySqlConnection.prototype.getArgumentPlaceholder = function (_) {
        return "?";
    };
    MySqlConnection.prototype.init = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this;
                        return [4 /*yield*/, promise_1.default.createPool({
                                connectionLimit: this.connectionPoolLimit,
                                host: this.dbUrl.hostname,
                                user: decodeURIComponent(this.dbUrl.username),
                                password: decodeURIComponent(this.dbUrl.password),
                                database: this.getDatabase(),
                                port: Number(this.dbUrl.port),
                            })];
                    case 1:
                        _a.pool = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return MySqlConnection;
}());
exports.MySqlConnection = MySqlConnection;
//# sourceMappingURL=mysql-connection.js.map